@@ -3,35 +3,38 @@
  *  Module    : xref.c
  *  Author    : I. Lea & H. Brugge
  *  Created   : 1993-07-01
- *  Updated   : 2016-06-02
+ *  Updated   : 2025-11-29
  *  Notes     :
  *
- * Copyright (c) 1993-2016 Iain Lea <iain@bricbrac.de>
+ * Copyright (c) 1993-2026 Iain Lea <iain@bricbrac.de>
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
+ *
+ * 1. Redistributions of source code must retain the above copyright notice,
+ *    this list of conditions and the following disclaimer.
+ *
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. The name of the author may not be used to endorse or promote
- *    products derived from this software without specific prior written
- *    permission.
  *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
- * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
- * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
- * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
- * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
- * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
- * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
- * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
- * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * 3. Neither the name of the copyright holder nor the names of its
+ *    contributors may be used to endorse or promote products derived from
+ *    this software without specific prior written permission.
+ *
+ * * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
+ * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
+ * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
+ * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
+ * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
+ * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
+ * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
+ * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
  */
 
 
@@ -62,13 +65,18 @@ open_overview_fmt_fp(
 		if (!*nntp_caps.over_cmd)
 			return (FILE *) 0;
 
-		if ((nntp_caps.type == CAPABILITIES && nntp_caps.list_overview_fmt) || nntp_caps.type != CAPABILITIES)
+		if (nntp_caps.list_overview_fmt)
 			return (nntp_command("LIST OVERVIEW.FMT", OK_GROUPS, NULL, 0));
 
 		return (FILE *) 0;
 	}
 #endif /* NNTP_ABLE */
+
+#ifndef NNTP_ONLY
 	return (fopen(overviewfmt_file, "r"));
+#else
+	return (FILE *) 0;
+#endif /* !NNTP_ONLY */
 }
 
 
@@ -80,8 +88,7 @@ overview_xref_support(
 	void)
 {
 	FILE *fp;
-	char *ptr;
-	char *p, *q;
+	char *ptr, *p;
 	t_bool supported = FALSE;
 	size_t res_fields = 9; /* initial number of overview fields */
 	size_t fields = 0;
@@ -100,7 +107,7 @@ overview_xref_support(
 				debug_print_file("NNTP", "<<<%s%s", logtime(), ptr);
 #endif /* DEBUG && NNTP_ABLE */
 
-			fields++;
+			++fields;
 
 			/* expand overview fmt array */
 			if (fields >= res_fields) {
@@ -110,14 +117,14 @@ overview_xref_support(
 
 			if ((p = strchr(ptr, ':'))) {
 				if (p == ptr) { /* metadata items start with : */
-					/* currently there is only :lines ands :bytes reserved */
+					/* currently only :lines and :bytes are reserved */
 					if (!strcasecmp(ptr, ":lines")) {
 						ofmt[fields].type = OVER_T_INT;
 						ofmt[fields].name = my_strdup("Lines:");
 						if (fields != 7) {
 							expensive_over_parse = TRUE;
 #ifdef DEBUG
-							if (debug & DEBUG_NNTP)
+							if ((debug & DEBUG_NNTP) && verbose > 1)
 								debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 7);
 #endif /* DEBUG */
 						}
@@ -130,7 +137,7 @@ overview_xref_support(
 						if (fields != 6) {
 							expensive_over_parse = TRUE;
 #ifdef DEBUG
-							if (debug & DEBUG_NNTP)
+							if ((debug & DEBUG_NNTP) && verbose > 1)
 								debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 6);
 #endif /* DEBUG */
 						}
@@ -143,19 +150,18 @@ overview_xref_support(
 				/* optional items require :full */
 				if (!strcasecmp(p, ":full")) {
 					ofmt[fields].type = OVER_T_FSTRING;
-					q = strchr(p, ':');
-					*(++q) = '\0';
+					*(++p) = '\0';
 					ofmt[fields].name = my_strdup(ptr);
 					if (fields < 7) {
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected > %d", ptr, fields, 7);
 #endif /* DEBUG */
 					}
 #ifdef DEBUG
 					else {
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d", ptr, fields);
 					}
 #endif /* DEBUG */
@@ -163,14 +169,14 @@ overview_xref_support(
 					continue;
 				}
 
-				/* madatory items */
+				/* mandatory items */
 				if (!strcasecmp(ptr, "Subject:")) {
 					ofmt[fields].type = OVER_T_STRING;
 					ofmt[fields].name = my_strdup(ptr);
 					if (fields != 1) {
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 1);
 #endif /* DEBUG */
 					}
@@ -183,7 +189,7 @@ overview_xref_support(
 					if (fields != 2) {
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 2);
 #endif /* DEBUG */
 					}
@@ -196,7 +202,7 @@ overview_xref_support(
 					if (fields != 3) {
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 3);
 #endif /* DEBUG */
 					}
@@ -209,7 +215,7 @@ overview_xref_support(
 					if (fields != 4) {
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 4);
 #endif /* DEBUG */
 					}
@@ -222,7 +228,7 @@ overview_xref_support(
 					if (fields != 5) {
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 5);
 #endif /* DEBUG */
 					}
@@ -235,7 +241,7 @@ overview_xref_support(
 					if (fields != 6) {
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 6);
 #endif /* DEBUG */
 					}
@@ -248,7 +254,7 @@ overview_xref_support(
 					if (fields != 7) {
 						expensive_over_parse = TRUE;
 #ifdef DEBUG
-						if (debug & DEBUG_NNTP)
+						if ((debug & DEBUG_NNTP) && verbose > 1)
 							debug_print_file("NNTP", "OVERVIEW.FMT: %s at position %d expected %d", ptr, fields, 7);
 #endif /* DEBUG */
 					}
@@ -262,7 +268,7 @@ overview_xref_support(
 		TIN_FCLOSE(fp);
 	}
 
-	fields++;
+	++fields;
 	/* resize */
 	ofmt = my_realloc(ofmt, sizeof(struct t_overview_fmt) * (fields + 1));
 
@@ -272,8 +278,8 @@ overview_xref_support(
 
 	if (fields < 2) {
 #ifdef DEBUG
-		if (debug & DEBUG_NNTP)
-			debug_print_file("NNTP", fp ? "OVERVIEW.FMT: Empty response - using safe defaults" : "OVERVIEW.FMT: not advertized - using safe defaults");
+		if ((debug & DEBUG_NNTP) && verbose > 1)
+			debug_print_file("NNTP", fp ? "OVERVIEW.FMT: Empty response - using safe defaults" : "OVERVIEW.FMT: not advertised - using safe defaults");
 #endif /* DEBUG */
 		ofmt = my_realloc(ofmt, sizeof(struct t_overview_fmt) * (8 + 1));
 		ofmt[1].type = OVER_T_STRING;
@@ -316,7 +322,7 @@ overview_xref_support(
 	 *         (use nntp_caps.hdr_cmd in txt_warn_xref_not_supported ?)
 	 *       - if server doesn't mention XREF in LIST HEADERS
 	 */
-	if (read_news_via_nntp && !supported)
+	if (read_news_via_nntp && !supported && !strstr(serverrc.disabled_nntp_cmds, "LIST OVERVIEW.FMT") && !strstr(serverrc.disabled_nntp_cmds, "HDR"))
 		wait_message(2, _(txt_warn_xref_not_supported));
 
 	return supported;
@@ -345,33 +351,34 @@ art_mark_xref_read(
 
 	xref_ptr = art->xref;
 
-	/*
-	 * check sitename matches nodename of current machine (ignore for now!)
-	 */
-	while (*xref_ptr != ' ' && *xref_ptr)
-		xref_ptr++;
+	/* skip sitename */
+	while (*xref_ptr != ' ' && *xref_ptr != '\t' && *xref_ptr)
+		++xref_ptr;
 
 	/*
 	 * tokenize each pair and update that newsgroup if it is in my_group[].
 	 */
 	forever {
-		while (*xref_ptr == ' ')
-			xref_ptr++;
+		while (*xref_ptr == ' ' || *xref_ptr == '\t')
+			++xref_ptr;
 
 		groupname = xref_ptr;
-		while (*xref_ptr != ':' && *xref_ptr)
-			xref_ptr++;
+		while (*xref_ptr != ':' && *xref_ptr != ' ' && *xref_ptr != '\t' && *xref_ptr)
+			++xref_ptr;
 
 		if (*xref_ptr != ':')
 			break;
 
 		ptr = xref_ptr++;
-		artnum = atoartnum(xref_ptr);
-		while (isdigit((int) *xref_ptr))
-			xref_ptr++;
+		if ((artnum = atoartnum(xref_ptr)) <= 0)
+			break;
+		while (isdigit((unsigned char) *xref_ptr))
+			++xref_ptr;
 
 		if (&ptr[1] == xref_ptr)
 			break;
+		if (*xref_ptr && *xref_ptr != ' ' && *xref_ptr != '\t')
+			break;
 
 		c = *ptr;
 		*ptr = '\0';
@@ -392,7 +399,7 @@ art_mark_xref_read(
 
 		if (group && group->newsrc.xbitmap) {
 			if (artnum >= group->newsrc.xmin && artnum <= group->xmax) {
-				if (!((NTEST(group->newsrc.xbitmap, artnum - group->newsrc.xmin) == ART_READ) ? TRUE : FALSE)) {
+				if (NTEST(group->newsrc.xbitmap, artnum - group->newsrc.xmin) != ART_READ) {
 					NSET0(group->newsrc.xbitmap, artnum - group->newsrc.xmin);
 					if (group->newsrc.num_unread > 0)
 						group->newsrc.num_unread--;
@@ -400,8 +407,8 @@ art_mark_xref_read(
 					if (debug & DEBUG_NEWSRC) {
 						debug_mesg = fmt_string("FOUND!Xref: [%s:%"T_ARTNUM_PFMT"] marked READ num_unread=[%"T_ARTNUM_PFMT"]",
 							groupname, artnum, group->newsrc.num_unread);
-							debug_print_comment(debug_mesg);
-							debug_print_bitmap(group, NULL);
+						debug_print_comment(debug_mesg);
+						debug_print_bitmap(group, NULL);
 /*						error_message(2, debug_mesg); */
 						free(debug_mesg);
 					}
@@ -423,17 +430,11 @@ NSETRNG1(
 	t_artnum low,
 	t_artnum high)
 {
-	t_artnum i;
-
-	if (bitmap == NULL) {
-#ifdef DEBUG
-		error_message(2, "NSETRNG1() failed. Bitmap == NULL");
-#endif /* DEBUG */
-		return;
-	}
+	assert(((void) "NSETRNG1() failed. bitmap == NULL", bitmap != NULL));
 
 	if (high >= low) {
 		if (NOFFSET(high) == NOFFSET(low)) {
+			t_artnum i;
 			for (i = low; i <= high; i++) {
 				NSET1(bitmap, i);
 			}
@@ -442,7 +443,7 @@ NSETRNG1(
 			if (NOFFSET(high) > NOFFSET(low) + 1)
 				memset(&bitmap[NOFFSET(low) + 1], NBITSON, (size_t) (NOFFSET(high) - NOFFSET(low) - 1));
 
-			BIT_OR(bitmap, high, ~ (NBITNEG1 << NBITIDX(high)));
+			BIT_OR(bitmap, high, ~(NBITNEG1 << NBITIDX(high)));
 		}
 	}
 }
@@ -457,15 +458,11 @@ NSETRNG0(
 	t_artnum low,
 	t_artnum high)
 {
-	t_artnum i;
-
-	if (bitmap == NULL) {
-		error_message(2, "NSETRNG0() failed. Bitmap == NULL");
-		return;
-	}
+	assert(((void) "NSETRNG0() failed. bitmap == NULL", bitmap != NULL));
 
 	if (high >= low) {
 		if (NOFFSET(high) == NOFFSET(low)) {
+			t_artnum i;
 			for (i = low; i <= high; i++) {
 				NSET0(bitmap, i);
 			}
